/* TOM default configuration
   Written by Pieter J. Schoenmakers <tiggr@gerbil.org>

   Copyright (C) 1996-1998 Pieter J. Schoenmakers.

   This file is part of TOM.  TOM is distributed under the terms of the
   TOM License, a copy of which can be found in the TOM distribution; see
   the file LICENSE.

   $Id: default.h,v 1.87 2000/10/24 00:50:21 bruce Exp $  */

/* This configuration file defines default values of all configuration
   settings needed by TOM.  For any machine you'll need to include this
   file and override the values which need to have different values.  */
#include <config/config.h>

/* Define this to 0 when this target does not employ os/os.c.  */
#ifndef HAVE_OS_OS_C
#define HAVE_OS_OS_C  1
#endif

/* The stack traversal policy used by the runtime; this affects code
   output by the compiler.  If the value is SP_PROTECT, all stack objects
   are considered root objects, and they are ensured to be gray during the
   protection phase.  It also implies that, in the context of incremental
   garbage collection, every assignment to a local reference var must
   ensure that the object is grayed.  With SP_MARK protection, the stack
   is marked, turning white objects gray, every time the number of gray
   objects reaches zero during the marking pahse of the garbage collector.
   This implies that, irrespective of the garbage collection policy, the
   check that an object being assigned to a stack variable is gray, can be
   omitted.  */
#ifndef STACK_PROTECT
#define STACK_PROTECT	SP_MARK
#endif
#define SP_PROTECT	0
#define SP_MARK		1

/* Iff !0, stack protection structs are needed to walk the stack.  Iff 0,
   object chunks are allocated from large ranges of memory, to be able to
   test for something found on the stack being a pointer to an object.
   These ranges are allocated from the OS by mmap, vm_alloc or similar
   calls.  */
#ifndef STACK_REF_STRUCT
#define STACK_REF_STRUCT  0
#endif

/* Iff !0, a pointer to each extension struct is cached in a variable
   which is local to the method referencing object variables from that
   extension.  This causes better code to be generated for methods
   referencing more than once a variable from said extension struct.  */
#ifndef CACHE_EXTENSION_POINTER
#define CACHE_EXTENSION_POINTER  1
#endif

/* The default load path.  */
#ifndef DEFAULT_LOAD_PATH
#define DEFAULT_LOAD_PATH	1, PATH_PREFIX @"/share/tom"
#endif

/* Iff !0, malloc is thread safe (or we can't be multi-threaded, which is
   the same).  The default is 1, i.e. YES, we're thread safe.  */
#ifndef MALLOC_THREAD_SAFE
#define MALLOC_THREAD_SAFE  1
#endif

/* Iff !0, mmap, when used, does not grok MAP_FIXED very well.  */
#ifndef MMAP_NOT_FIXED
#define MMAP_NOT_FIXED  0
#endif

/* Iff !0, use  valloc for allocating chunk ranges.  */
#ifndef CHUNK_RANGE_VALLOC
#define CHUNK_RANGE_VALLOC  0
#endif

/* Macros to fetch from a va_list AP the next entity with the indicated
   type.  Needed only for smaller-than-int and smaller-than-double types
   (which normally get promoted).  */
#ifndef NEED_VA_ARG_WORKAROUND
#ifndef VA_ARG_BYTE
#define VA_ARG_BYTE(AP)  (va_arg ((AP), tom_byte))
#endif
#ifndef VA_ARG_CHAR
#define VA_ARG_CHAR(AP)  (va_arg ((AP), tom_char))
#endif
#ifndef VA_ARG_FLOAT
#define VA_ARG_FLOAT(AP)  (va_arg ((AP), tom_float))
#endif
#else
/* Support gcc 2.96 (and later?)  */
#ifndef VA_ARG_BYTE
#define VA_ARG_BYTE(AP)  (va_arg ((AP), tom_int))
#endif
#ifndef VA_ARG_CHAR
#define VA_ARG_CHAR(AP)  (va_arg ((AP), tom_int))
#endif
#ifndef VA_ARG_FLOAT
#define VA_ARG_FLOAT(AP)  (va_arg ((AP), tom_double))
#endif
#endif

/* The type passed to __builtin_return.  */
#ifndef BUILTIN_RETURN_TYPE
#define BUILTIN_RETURN_TYPE  \
union				\
{				\
  struct { tom_int i; } i;	\
  struct { tom_long l; } l;	\
  struct { void *p; } p;	\
  struct { tom_double d; } d;	\
  struct { tom_float f; } f;	\
}
#endif

/* Macros to set and retrieve the difficult types from a
   builtin_return_type thing.  Currently only needed for float and double.  */
#ifndef RETURN_RETRIEVE_DOUBLE
#define RETURN_RETRIEVE_DOUBLE(RESULT)  ((RESULT)->d.d)
#endif
#ifndef RETURN_RETRIEVE_FLOAT
#define RETURN_RETRIEVE_FLOAT(RESULT)  ((RESULT)->f.f)
#endif
#ifndef RETURN_SET_DOUBLE
#define RETURN_SET_DOUBLE(RESULT, VALUE)  ((RESULT)->d.d = (VALUE))
#endif
#ifndef RETURN_SET_FLOAT
#define RETURN_SET_FLOAT(RESULT, VALUE)  ((RESULT)->f.f = (VALUE))
#endif

/* Parts of constructing an invocation.
   Perform any wrapping up of the ARGS needed to perform the actual
   application.  */
#ifndef APPLY_ARGS_COMPLETE
#define APPLY_ARGS_COMPLETE()  \
      do ; while (0)
#endif

#ifndef APPLY_ARGS_CONTINUE
#define APPLY_ARGS_CONTINUE()  \
      do ; while (0)
#endif

/* APPLY_ARGS_ARGS, APPLY_ARGS_APPLY and APPLY_ARGS_RETURN correspond to
   gcc's __builtin_apply_args, __builtin_apply, and __builtin_return.  The
   extra RET argument passed to APPLY_ARGS_APPLY should be a pointer to a
   builtin_return_value to be (possibly) used to store the result of the
   apply.

   If builtin_return_value values are forged by something other than
   APPLY_ARGS_APPLY, they should be BZERO'ed first.  Otherwise things will
   fail in mysterious ways on the i386.  */
#ifndef APPLY_ARGS_ARGS
#define APPLY_ARGS_ARGS  \
  __builtin_apply_args
#endif

#ifndef APPLY_ARGS_APPLY
#define APPLY_ARGS_APPLY(FUNC, ARGS, SIZE, RET)	 \
  __builtin_apply ((FUNC), (ARGS), (SIZE))
#endif

#ifndef APPLY_ARGS_RETURN
#define APPLY_ARGS_RETURN(RESULT)  \
  __builtin_return (RESULT)
#endif

/* Call bzero, or whatever is faster.  Default calls bzero.  */
#ifndef BZERO
#define BZERO(A, N)  bzero ((A), (N))
#endif

/* Now how do we stringify the actual value of the macro passed as X?  */
#define GETSTR(Y)	#Y

/* The name of the module information of a dynamically loaded object.  */
#define TRT_MODULE_INFO		"trt_module_info"

/********** memory allocation **********/

/* (trt/alloc.c) !0 iff chunk ranges are (used and) aligned at their size.  */
#ifndef RANGES_ARE_ALIGNED
#define RANGES_ARE_ALIGNED  HAVE_VM_ALLOCATE
#endif

/* (trt/alloc.c) Return !0 iff the ADDRESS is contained in the RANGE.  */
#ifndef ADDRESS_IN_RANGE
#if RANGES_ARE_ALIGNED
#define ADDRESS_IN_RANGE(ADDR, RANGE)  \
  (((POINTER_INT_TYPE) (ADDR) & (RANGE)->mask)	\
   == (POINTER_INT_TYPE) (RANGE)->first)
#else
#define ADDRESS_IN_RANGE(ADDR, RANGE)  \
  ((ADDR) > (char *) (RANGE)->first					\
   && (ADDR) < ((char *) (RANGE)->first + (RANGE)->num * vm_page_size))
#endif
#endif

/* (trt/alloc.c) The size of chunk ranges, in bytes.  If
   RANGES_ARE_ALIGNED, this should be a power of 2.  */
#ifndef RANGE_SIZE
#define RANGE_SIZE  (2 * 1024 * 1024)
#endif

/* (trt/alloc.c) The first address at which to try to place a chunk range.
   If this is large, and a power-of-two multiple of RANGE_SIZE, chunk
   ranges can actually grow without the need for extra chunks.  This is
   only important if RANGES_ARE_ALIGNED.  */
#ifndef FIRST_RANGE_ADDRESS
#define FIRST_RANGE_ADDRESS  0
#endif

/********** dynamic loading **********/

/* Initialize dynamic loading.  */
void dload_initialize ();

/* Load from the file NAME, returning its handle.  */
void *dload_load (char *name);

/* Lookup the symbol named NAME from the object with the HANDLE.  */
void *dload_lookup (void *handle, char *name);

/* Return the error message resulting from the previous dynamic loading
   operation.  */
char *dload_error ();

#ifndef DLOAD_SYMBOL_PREFIX
#define DLOAD_SYMBOL_PREFIX ""
#endif

/********** debugging **********/

#ifndef DEBUG_MESSAGE_MONITORING
#define DEBUG_MESSAGE_MONITORING  1
#endif

#ifndef NO_MALLOC_STATS
#define NO_MALLOC_STATS  1
#endif

#ifndef HAVE_UNIX98_PTHREADS
#define pthread_mutexattr_settype(attr, kind) \
  pthread_mutexattr_setkind_np(attr, kind)
#define PTHREAD_MUTEX_NORMAL PTHREAD_MUTEX_FAST_NP
#define PTHREAD_MUTEX_RECURSIVE PTHREAD_MUTEX_RECURSIVE_NP
#endif
